﻿--[[
Name: LibAvion-2.0
Revision: 40
Author(s): Sirow (EU-Gorgonnash)
Documentation: http://www.wowace.com/wiki/LibAvion-2.0
SVN: http://svn.wowace.com/wowace/trunk/LibAvion-2.0
Description: A library that triggers visual effects.
Dependencies: LibStub, LibSimpleTimer-1.0
]]


-------------------------------------------------------------------------------
-- Library - Version Info & Dependencies
-------------------------------------------------------------------------------
-- Version
local MAJOR 		= "LibAvion-2.0"
local MINOR 		= tonumber(("40"):match("%d+")) or 0

-- Dependencies
if not LibStub 						  then error(MAJOR .. " requires LibStub.") end
if not LibStub("LibSimpleTimer-1.0")  then error(MAJOR .. " requires LibSimpleTimer-1.0.") end


-------------------------------------------------------------------------------
-- Library - Register
-------------------------------------------------------------------------------
-- Register
local LibAvion 	= LibStub:NewLibrary(MAJOR, MINOR)
if not LibAvion then return end

-- Embed CallbackHandler and LibSimpleTimer
LibStub("LibSimpleTimer-1.0"):Embed(LibAvion)


-------------------------------------------------------------------------------
-- Library - Variables
-------------------------------------------------------------------------------
-- Set embeeding variables
LibAvion.mixinTargets = LibAvion.mixinTargets or {}
local mixins = {
	"TriggerAnimation",
	"StopAnimation",
	"StopAllAnimations",
	"IsAnimationRunning",
	
	"ScheduleTimer", 
	"ScheduleRepeatingTimer", 
	"IsTimerScheduled", 
	"CancelTimer",
	
	"animationCache",
}

-- Common
_G.LibAvion__updateInterval 	= 0.05
local updateInterval			= _G.LibAvion__updateInterval

-- Cache for animations/guides
LibAvion.animationCache	= LibAvion.animationCache 	or {}
LibAvion.caches			= LibAvion.caches 			or {
	animationFrames 	= {},
	animationCounter	= 0,
}
local animationCache	= LibAvion.animationCache
local animationFrames	= LibAvion.caches.animationFrames
local animationCounter	= LibAvion.caches.animationCounter


-------------------------------------------------------------------------------
-- Library - Utility functions
-------------------------------------------------------------------------------
--[[
local new, del, deepcopy
do
	-- Table with weak keys
	local cache = setmetatable({}, {__mode = 'k'})
	
	
	-- Create new table resuing cache
	function new()
		local t = next(cache)
		
		if t then
			cache[t] = nil
			return t
		else
			return {}
		end
	end
	
	
	-- Delete table
	function del(t)
		for k in pairs(t) do
			t[k] = nil
		end
		cache[t] = true
		
		return nil
	end
	
	
	-- Create a deep copy of table (Keep same metatable)
	function deepcopy(object)
	    local lookup_table = {}
		
	    local function _copy(object)
	        if type(object) ~= "table" then
	            return object
	        elseif lookup_table[object] then
	            return lookup_table[object]
	        end
			
	        local new_table = {}
	        lookup_table[object] = new_table
			
	        for index, value in pairs(object) do
	            new_table[_copy(index)] = _copy(value)
	        end
			
	        return setmetatable(new_table, getmetatable(object))
	    end
		
	    return _copy(object)
	end
end
]]--


-------------------------------------------------------------------------------
-- Library - Animation - Core
-------------------------------------------------------------------------------
-- Create a new frame
local function MakeFrame()
	-- Name of the frame
	local frameName = "Avion_Frame_" .. animationCounter

	-- Create a new frame and apply some basic settings
	local frame = CreateFrame("FRAME", frameName, UIParent)
	frame:Hide()
	frame:SetPoint("CENTER", UIParent, "CENTER")
	frame:SetBackdropColor(0, 0, 0, 0)
	frame:SetFrameStrata("BACKGROUND")
	frame:SetScale(1)
	
	-- Add a texture to the frame (Blend mode = additive)
	local texture = frame:CreateTexture(frameName .. "_Texture", "BACKGROUND")
	texture:SetParent(frame)
	texture:SetAllPoints(frame)
	texture:SetBlendMode("ADD")
	
	-- Add everything to the cache
	animationFrames[animationCounter] = {
		inUse = true,
		frame = frame,
		texture = texture,
		elapsedTimes = 0
	}
	
	-- Increment number of frames and return new number
	animationCounter = animationCounter + 1
	return (animationCounter-1)
end


-- Get an unused frame or create a new one (only ID)
local function GetAvailableFrameID()
	-- Is there any unused/invisible frame?
	for index, cache in pairs(animationFrames) do
		if (not cache.inUse) then
			-- Enable inUse and return index
			cache.inUse = true
			cache.elapsedTimes = 0
			
			return index
		end
	end
	
	-- If not, create one!
	return MakeFrame()
end


-- Get an unused frame or create a new one
local function GetAvailableFrame()
	local frameID = GetAvailableFrameID()
	return animationFrames[frameID], frameID
end


-- Update values with linear interporlation between start and end
local function Interpolation(startValues, endValues, param) 
	return startValues + (endValues - startValues) * param
end


-- Will rotate texture (and shrink it by a factor of sqrt(2)) and mirror at X/Y axis if needed
local function RotateAndMirror(texture, angle, mirrorX, mirrorY)
	--[[
	NOTE 1: 
	You want to rotate the rectangle (0,0), (1,0), (1,1), (0,1) arround the Point (0.5, 0.5).
	The normal 2D-Rotationsmatrix
	R = [cos(a) -sin(a); sin(a) cos(a)] (where ; means new row)
	will rotate everything around the origin point (0,0).
	Thus we need to do the following:
	Translate our desired rotation-point (0.5,0.5) into the origin by
	T = [1 0 -0.5; 0 1 -0.5; 0 0 1] (in homogenous coords, rougly meaning [x y] will get [x y 1])
	Now we can use the above R (also in homogenous coords) to rotate around the origin and then use
	T^-1 = [1 0 0.5; 0 1 0.5; 0 0 1] 
	to move everything back to were it belongs.
	Given our texture coords [x y]_texture this would lead to:
	[x y 1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * [cos(a) -sin(a) 0; sin(a) cos(a) 0; 0 0 1]  * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture
	which (in short) can be seen below.
	
	NOTE 2:
	There is just one tiny problem, after each rotation the coordibates still have to be
	inside the rectangle (0,0), (1,0), (1,1), (0,1)  or some parts of the texture won't be
	displayed. Thus, the all texture coords (texture NOT frame) are "scaled down"
	by a factor of sqrt(2) (which is, after pytagoras the length of the recangles diagonal).
	Yet scaling is also done around the origin, not (0.5, 0.5), thus we used the exact same
	procedure as above, only replacing R with
	S = [1/sqrt 0 0; 0 1/sqrt(2) 0; 0 0 1]
	leaving:
	[x y 1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * [1/sqrt 0 0; 0 1/sqrt(2) 0; 0 0 1]  * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture
	
	NOTE 3:
	Mirroring is done by
	M_x = [-1 0 0; 0 1 0; 0 0 1]
	M_y = [1 0 0; 0 -1 0; 0 0 1]
	Some rules as above apply to mirroring. Need to mirror around X=0.5 or Y=0.5 axis, but those matrizes only
	mirror around X- or Y-axis, leading to:
	[x y1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * M_x * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture (X Axis)
	[x y1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * M_y * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture (Y Axis)
	[x y1]_transformed = [1 0 0.5; 0 1 0.5; 0 0 1]  * M_x * M_y * [1 0 -0.5; 0 1 -0.5; 0 0 1] *[x y 1]_texture (X=Y Axis)
	
	NOTE 4:
	For final result, all transformations need to be combined (in the right order, since matrix multiplication
	is commonly not kummulativ):
	[x y1]_transformed 
	= [1 0 0.5; 0 1 0.5; 0 0 1]  * [1/sqrt 0 0; 0 1/sqrt(2) 0; 0 0 1]  * [1 0 -0.5; 0 1 -0.5; 0 0 1]  (scale)
	* [1 0 0.5; 0 1 0.5; 0 0 1]  * [cos(a) -sin(a) 0; sin(a) cos(a) 0; 0 0 1]  * [1 0 -0.5; 0 1 -0.5; 0 0 1]  (rotate)
	* [1 0 0.5; 0 1 0.5; 0 0 1]  * M_xORy * [1 0 -0.5; 0 1 -0.5; 0 0 1]  (mirror)
	* [x y 1]_texture
	Which is equal to:
	x y1]_transformed 
	= [1 0 0.5; 0 1 0.5; 0 0 1]  (move back from origin)
	* [1/sqrt 0 0; 0 1/sqrt(2) 0; 0 0 1]  (scale)
	* [cos(a) -sin(a) 0; sin(a) cos(a) 0; 0 0 1]  (rotate)
	* M_xORy (mirror)
	* [1 0 -0.5; 0 1 -0.5; 0 0 1]  (move to origin)
	* [x y 1]_texture
	Apply this transformations to all edges of the (0,0), (1,0), (1,1), (0,1) rectangle, you SHOULD
	get the values you see below. (0.5 * sqrt(2) = 1/sqrt(2) ~ 0.7072)
	
	NOTE 5:
	Remember that texture origin is at top-left of frame. Meaning x goes right, y goes down.
	]]
	
	-- Mirror
	local mX, mY = 1, 1
	if (mirrorX) then
		mX = -1
	end
	if (mirrorY) then
		mY = -1
	end
	
	-- Rotation
	local cosAlpha = 0.7072 * math.cos(angle)
	local sinAlpha = 0.7072 * math.sin(angle)
	texture:SetTexCoord(
		-cosAlpha*mY + sinAlpha*mX + 0.5, -sinAlpha*mY - cosAlpha*mX + 0.5, 
		-cosAlpha*mY - sinAlpha*mX + 0.5, -sinAlpha*mY + cosAlpha*mX + 0.5, 
		 cosAlpha*mY + sinAlpha*mX + 0.5,  sinAlpha*mY - cosAlpha*mX + 0.5, 
		 cosAlpha*mY - sinAlpha*mX + 0.5,  sinAlpha*mY + cosAlpha*mX + 0.5
	)
end


-- Initialize frame settings and show it
local function InitFrame(object, animationTable)
	-- (Re)set texture file and color
	object.texture:SetTexture(animationTable.textureFile)
	object.texture:SetVertexColor(animationTable.color.r, animationTable.color.g, animationTable.color.b)
	
	-- Adjust frame size, position and alpha
	object.frame:SetHeight(animationTable.size.height * 1.4142)
	object.frame:SetWidth(animationTable.size.width * 1.4142)
	object.frame:SetPoint("CENTER", UIParent, "CENTER", animationTable.position.x, animationTable.position.y)
	object.frame:SetAlpha(animationTable.alpha)
	
	-- Rotation
	RotateAndMirror(object.texture, animationTable.rotation, animationTable.mirror.x, animationTable.mirror.y)
	
	-- Finally show it
	object.frame:Show()
end


-- Actually update frame animation
local function AnimateFrame(object, animationTable, param)
	-- Get and set updated values
	if (animationTable.isAnimated.position) then
		local position = {
			x = Interpolation(
				animationTable.position.x, 
				animationTable.animation.position.x, 
				animationTable.animationFunc.position(param, animationTable.addRandom.position)
			),
			y = Interpolation(
				animationTable.position.y, 
				animationTable.animation.position.y, 
				animationTable.animationFunc.position(param, animationTable.addRandom.position)
			)
		}
		object.frame:SetPoint("CENTER", UIParent, "CENTER", position.x, position.y)
	end
	if (animationTable.isAnimated.scale) then
		local scale = Interpolation(
			1, 
			animationTable.animation.scale, 
			animationTable.animationFunc.scale(param, animationTable.addRandom.scale)
		)
		object.frame:SetHeight(animationTable.size.height * 1.4142 * scale)
		object.frame:SetWidth(animationTable.size.width * 1.4142 * scale)
	end
	if (animationTable.isAnimated.color) then
		local color = {
			r = Interpolation(
				animationTable.color.r, 
				animationTable.animation.color.r, 
				animationTable.animationFunc.color(param, animationTable.addRandom.color)
			),
			g = Interpolation(
				animationTable.color.g, 
				animationTable.animation.color.g, 
				animationTable.animationFunc.color(param, animationTable.addRandom.color)
			),
			b = Interpolation(
				animationTable.color.b, 
				animationTable.animation.color.b, 
				animationTable.animationFunc.color(param, animationTable.addRandom.color)
			)
		}
		object.texture:SetVertexColor(color.r, color.g, color.b)
	end
	if (animationTable.isAnimated.rotation) then
		local angle = Interpolation(
			animationTable.rotation, 
			animationTable.animation.rotation,
			animationTable.animationFunc.rotation(param, animationTable.addRandom.rotation)
		)
		RotateAndMirror(object.texture, angle, animationTable.mirror.x, animationTable.mirror.y)
	end	
end


-- Update frame animation
local function UpdateAnimation(object, animationTable, forceUpdate, fadeOutNow)
	-- Update elapsed time (time since animation was triggered)
	local elapsedTime = object.elapsedTimes * updateInterval
	object.elapsedTimes = object.elapsedTimes + 1
	
	-- Parameter for update function
	local param
	
	-- Animate frame, move, resize, color, rotate, ect.
	if (elapsedTime <= animationTable.animationTime) then
		-- Calc parameter for linear interpolation (param \in [0...1]) and animate frame
		if (animationTable.fadeMode == 1) then
			param = elapsedTime / (animationTable.animationTime + animationTable.fadeTime)
		else
			param = elapsedTime / animationTable.animationTime
		end
		AnimateFrame(object, animationTable, param)
		
	-- Stop doing anything to the frame
	elseif (animationTable.noEnd and not fadeOutNow) then
		-- We just disabled animation, so no update to actual animation time
		object.elapsedTimes = object.elapsedTimes - 1
		
		-- Stop animation timer, since nothing to do
		if (LibAvion:IsTimerScheduled("Avion_UpdateAnimation_" .. object.triggerID)) then
			LibAvion:CancelTimer("Avion_UpdateAnimation_" .. object.triggerID)
		end
	
	-- Start fading frame
	elseif(elapsedTime - animationTable.animationTime <= animationTable.fadeTime) then	
		-- Calc parameter for linear interpolation (param \in [0...1]) and continue animation
		if (animationTable.fadeMode == 1) then
			param = elapsedTime / (animationTable.animationTime + animationTable.fadeTime)
			
			AnimateFrame(object, animationTable, param)
		-- Run animation in reverse order
		elseif (animationTable.fadeMode == 2) then
			param = 1 - (elapsedTime - animationTable.animationTime) / animationTable.fadeTime
			
			AnimateFrame(object, animationTable, param)
		elseif (forceUpdate) then
			AnimateFrame(object, animationTable, 1)
		end

		-- Fade frame out
		param = (elapsedTime - animationTable.animationTime) / animationTable.fadeTime
		object.frame:SetAlpha(
			Interpolation(
				animationTable.alpha, 
				0, 
				animationTable.animationFunc.alpha(param, animationTable.addRandom.alpha)
			)
		)
	
	-- Done, Unregister
	else
		LibAvion:StopAnimation(object.triggerID, false)
	end
end


-- "Register" a new frame for animation (this registration handels timing)
local function StartAnimationTimer(object, animationTable, updatedAnimationTime, updatedWaitTime)
	local self = LibAvion
	
	-- If there is NO wait-time, initialize and run animation
	if (updatedWaitTime == 0) then
		-- Initialize frame stuff
		if (not object.frame:IsVisible()) then
			InitFrame(object, animationTable)
		end
		
		-- We only need to test if something is animated if there is really animation-time ^^
		local somethingAnimated = false
		if (updatedAnimationTime ~= 0) then
			-- If there is nothing to animate, well ... life is easier ^^
			for _, value in pairs(animationTable.isAnimated) do
				if (value) then
					somethingAnimated = true
					break
				end
			end
		end
		
		-- Well ... thats life, go the hard way ^^
		if (somethingAnimated or updatedAnimationTime == 0) then
			-- Register animation
			LibAvion:ScheduleRepeatingTimer(
				"Avion_UpdateAnimation_" ..object.triggerID,
				UpdateAnimation,
				updateInterval,
				--self, 
				object, 
				animationTable,
				false,
				false
			)
		else
			-- Start animation after animationTime has passed, since there was nothing to animate...
			LibAvion:ScheduleTimer(
				"Avion_DisplayAnimation_" .. object.triggerID,
				StartAnimationTimer,
				animationTable.animationTime, 
				--self, 
				object, 
				animationTable, 
				0, 
				0
			)
		end
	-- If there IS wait-time, retrigger after waitTime-delay
	else
		-- Show frame allready?
		if (animationTable.showWhileWait) then
			InitFrame(object, animationTable)
		end
		
		-- Start waiting
		LibAvion:ScheduleTimer(
			"Avion_WaitAnimation_" .. object.triggerID,
			StartAnimationTimer,
			animationTable.waitTime,  
			--self, 
			object, 
			animationTable, 
			animationTable.animationTime, 
			0
		)
	end
end


-------------------------------------------------------------------------------
-- Library - Animation - API
-------------------------------------------------------------------------------
--[[
######################################################
# :TriggerAnimation(triggerID, animationTable)
# Notes:
#	Start showing given (by animationTable) animation and associate it with identifier triggerID
# Arguments:
#	triggerID (string) - The unique identifier for this animation.
#	animationTable (table) - The table that defines this animation, see EOF for more.
# Returns:
######################################################
]]--
function LibAvion:TriggerAnimation(triggerID, animationTable)
	-- Quite when disabled or allready registered
	if (self:IsAnimationRunning(triggerID)) then
		return
	end
	
	-- Make things local
	local object, frameID
	object, frameID = GetAvailableFrame()
	
	-- Backup animation data (eg. needed when fading out)
	animationCache[triggerID] = {
		animationTable = animationTable,
		frameID = frameID
	}
	
	-- Save which triggerID triggerd this animation
	object.triggerID = triggerID
	
	-- Play soundfile if there is one to play
	if (animationTable.soundFile) then
		PlaySoundFile(animationTable.soundFile)
	end
	
	-- Finally "start" animation/timer
	StartAnimationTimer(object, animationTable, animationTable.animationTime, animationTable.waitTime)
end


--[[
######################################################
# :StopAnimation(triggerID, fadeOut)
# Notes:
#	Stop the animation that is assigned to this identifier.
# Arguments:
#	triggerID (string) - The unique identifier for this animation.
#	fadeOut - (bool) - Wether or not to fade or just stop and hide animation.
# Returns:
######################################################
]]--
function LibAvion:StopAnimation(triggerID, fadeOut)
	-- Hmmm, well trying to unregister an unregister animation may be painful >:>
	if (self:IsAnimationRunning(triggerID)) then
		-- Unregister animation (delayed and repeating animations)
		if (self:IsTimerScheduled("Avion_UpdateAnimation_" .. triggerID)) then
			self:CancelTimer("Avion_UpdateAnimation_" .. triggerID)
		end
		if (self:IsTimerScheduled("Avion_DisplayAnimation_" .. triggerID)) then
			self:CancelTimer("Avion_DisplayAnimation_" .. triggerID)
		end
		if (self:IsTimerScheduled("Avion_WaitAnimation_" .. triggerID)) then
			self:CancelTimer("Avion_WaitAnimation_" .. triggerID)
		end
		
		-- Smoothly fade-out frame
		if (fadeOut) then
			-- Needed vars
			local object 			= animationFrames[animationCache[triggerID].frameID]
			local animationTable	= animationCache[triggerID].animationTable
			
			-- Adjust run-time of this animation to fit fading
			if (object.elapsedTimes * updateInterval - animationTable.animationTime < animationTable.FadeTime) then
				object.elapsedTimes = animationTable.animationTime / updateInterval
			end
			
			-- Register for fading animation
			self:ScheduleRepeatingTimer(
				"Avion_UpdateAnimation_" .. triggerID,
				UpdateAnimation, 
				updateInterval,
				--self, 
				object, 
				animationTable,
				false,
				true
			)
		
		-- Stop and hide animation
		else
			-- Hide frame and its not inUse anymore...
			animationFrames[animationCache[triggerID].frameID].texture:SetTexture("Interface\\Icons\\Temp.blp")
			animationFrames[animationCache[triggerID].frameID].frame:Hide()
			animationFrames[animationCache[triggerID].frameID].inUse = false
			
			-- Delete frame from animation cache
			animationCache[triggerID].animationTable 	= nil
			animationCache[triggerID] 				= nil
		end
	end
end


--[[
######################################################
# :StopAllAnimations(fadeOut)
# Notes:
#	Stops all running animations.
# Arguments:
#	fadeOut - (bool) - Wether or not to fade or just stop and hide animation.
# Returns:
######################################################
]]--
function LibAvion:StopAllAnimations(fadeOut)
	if (animationCache) then
		for triggerID, _ in pairs(animationCache) do
			self:StopAnimation(triggerID, fadeOut)
		end
	end
end


--[[
######################################################
# :IsAnimationRunning(triggerID)
# Notes:
#	Test if animation assosiated with this identifier is still active.
# Arguments:
#	triggerID (string) - The unique identifier for this animation.
# Returns:
#	true if its still active, false otherwise.
######################################################
]]--
function LibAvion:IsAnimationRunning(triggerID)
	if (animationCache and animationCache[triggerID]) then
		return true
	else
		return false
	end
end


--[[
######################################################
# :Embed(target)
# Notes:
#	Embeds methods listed in local <mixins> into target table.
# Arguments:
#	target (table) - The table on which to export methods onto.
# Returns:
#	The table provided, after embedding.
######################################################
]]--
function LibAvion:Embed(target)
	LibAvion.mixinTargets[target] = true
	
	for _, v in pairs(mixins) do
		target[v] = LibAvion[v]
	end
	
	return target
end


-- Reembed (eg. lib was updated)
for addon in pairs(LibAvion.mixinTargets) do
	LibAvion:Embed(addon)
end


-------------------------------------------------------------------------------
-- Library - Export
-------------------------------------------------------------------------------
_G.LibAvion__caches 			= LibAvion.caches
_G.LibAvion__GetAvailableFrame 	= GetAvailableFrame
_G.LibAvion__InitFrame			= InitFrame
_G.LibAvion__UpdateAnimation	= UpdateAnimation


-------------------------------------------------------------------------------
-- Dokumentation - Variables
-------------------------------------------------------------------------------
--[[
VAR: AnimationTable
TYPE: Table
DESC: It cantains all needed information to start an animation:
	{
		alpha 		= Alpha value when its shown (number)
		color 			= {
			r		= Red color when its shown (number)
			g 		= Green color when its shown (number)
			b 		= Blue color when its shown (number)
		}
		position 		= {
			x 		= X-Position on screen when its shown (number)
			y 		= Y-Position on screen when its shown (number)
		}
		size 			= {
			width 	= Width when its shown (number)
			height 	= Height when its shown (number)
		}
		rotation 		= Rotation of texture when its shown (number)
		isAnimated 		= {
			color 		= Should color be animated? (bool)
			position 	= Should position be animated? (bool)
			scale 		= Should scale be animated? (bool)
			rotation 	= Should rotation be animated? (bool)
		}
		animation 		= {
			color 		= {
				r 	= Red color that is beeing animated to (number)
				g 	= Greem color that is beeing animated to (number)
				b 	= Blue color that is beeing animated to (number)
			}, 
			position 	= {
				x 	= X-Position that is beeing animated to (number)
				y 	= Y-Position that is beeing animated to (number)
			}, 
			scale 		= Scale that is beeing animated to (number)
			rotation 	= Rotations of texture that is beeing animated to (number)
		}
		animationFunc	= {
			alpha 	= Smoothness/speed of alpha animation (function)
			color 		= Smoothness/speed of color animation (function)
			position 	= Smoothness/speed of position animation (function)
			scale 		= Smoothness/speed of scale animation (function)
			rotation 	= Smoothness/speed of rotation animation (function)
		}
NOTE: All of this functions HAVE to be pointers to function with head 
"function (param, rnd)", where <param> gonna be a value between 0 (start)
and 1 (end), depending on the time the animation is still running.
<rnd> is the value that can be set in the options dialogue under "Randomness",
its meant to add some random effect.
		addRandom 		= {
			alpha 	= Add some randomness to alpha animation (number)
			color 		= Add some randomness to color animation (number)
			position 	= Add some randomness to position animation (number)
			scale 		= Add some randomness to scale animation (number)
			rotation 	= Add some randomness to rotation animation (number)
		}
		fadeMode 		= How to fade the frame
NOTE: 0 means, just fading; 1 means fade and coninue animation, 
2 means inverting animation when fading out
		animationTime 	= The time the "animation" phase should take
		fadeTime 		= The time it should take the frame to fade 
					out after completion (number)
		waitTime 		= The time it should take from triggering the 
					animation until the actual animation should start 
NOTE: Can be used to create difficult multi-animations. (number)
		showWhileWait 	= Allready show the frame/animation while
					 "waitTime" is yet running (bool)
		textureFile 		= Texture file (file link, no SharedMedia!) to 
					the texture that should be used (string)
		soundFile 		= Sound file (file link, no SharedMedia!) to 
					the sound that is triggered as soon as the 
					animation starts (string)
		mirror 		= {
			x 		= Mirror texture on x-axis (bool)
			y 		= Mirror texture on y-axis (bool)
		}
		noEnd 		= The animation won't fade when its finished, but
					will rather need to be stopped with 
					LibAvion:StopAnimation(...) (bool)
}


VAR: updateInterval
TYPE: Number
DESC: Update rate for all animations, eg. 0.05 means roughly 25 updates per second.


VAR: animationCache
TYPE: Table
DESC: Stores all currently active (running) animations, more speficly the unique key of the
frame (in animationFrames), mostly refered to as the FrameID that was assossiated to 
this animation. The frame thats beeing animated. Also saves (a link to)  the table that's used to
define the animation.
This table is accessed by the unique key, that was used to start the animation, namely triggerID.


VAR: animationFrames 
Type: Table
DESC: Table that contains a pool of created/available frame objects to be used for visualising
an animation. It has "animationCounter" number of entrys and the unique key that is
used to access it is called frameID. (or animationFrameID)
Each entry inhabits the following entrys:
	.inUse		- Say if the frame is currently beeing used for visualising an animation
	.frame		- the actuell fram object that can be shown or hidden
	.texture		- the texture object
	.elapsedTimes	- iterater that (if the animation is currently running) tells how often it has been updated allready


VAR: animationCounter
TYPE: Number
DESC: Its just #animationFrames, nothing more, nothing less


EOF]]